var tabs = require("sdk/tabs");
var pageMod = require("sdk/page-mod");
var pageWorker = require("sdk/page-worker");
var panel = require("sdk/panel");
var self = require("sdk/self");
var data = require("sdk/self").data;
var preferences = require("sdk/simple-prefs").prefs;
var preferencesService = require("sdk/preferences/service");
var notifications = require("sdk/notifications");
var Request = require("sdk/request").Request;
var windows = require("sdk/windows");
var { viewFor } = require("sdk/view/core");

var { ToggleButton } = require("sdk/ui/button/toggle");
var { Cc, Ci } = require("chrome");

var _ = require("sdk/l10n").get;

var pps = Cc["@mozilla.org/network/protocol-proxy-service;1"]
			  .getService(Ci.nsIProtocolProxyService);

var configTimestamp = 0;
var unblock = false;
var proxyFilter = null;
var proxyList = [];
var hostnames = [];
var notifications = [];

function loadAddonConfig(reload = false) {
	var addonConfig = Request({
	  url: 'http://api.ytunblocker.com/addon-configuration.json?id=' + self.id + '&version=' + self.version,
	  onComplete: function (response) {
		var config = JSON.parse(response.text);  
		
		configTimestamp = Date.now();
		proxyList = [];
		
		if (config.proxyList) {
			for (var i = 0; i < config.proxyList.length; i++) {
				var proxy = config.proxyList[i];
				addProxy(proxy.host, proxy.port);
			}
		}
		
		if (config.hostnames) {
			hostnames = config.hostnames;
		}
		
		if (config.notifications) {
			notifications = config.notifications;
		}
		
		preferences.disableSupport = false;
		
		unregisterProxyFilter();
		registerProxyFilter();
		
		if (reload) {
			doUnblock(true);
		}
	  }
	});
	addonConfig.get();
}

function checkHost(host) {
	for (var i = 0; i < hostnames.length; i++) {
		if (host.indexOf(hostnames[i]) != -1) {
			return true;
		}
	}
	return false;
}

function registerProxyFilter() {
	proxyFilter = {
		applyFilter: function(pps, uri, proxy) {
			var activeProxy = proxy;
			
			if (checkHost(uri.host) && proxyList.length > 0) {
				activeProxy = proxyList[proxyList.length-1].proxyInfo;
			}
			
			if (unblock) {
				return activeProxy;
			} else {
				return proxy;
			}
		}
	};
	pps.registerFilter(proxyFilter, 1000);
}

function unregisterProxyFilter() {
	if (proxyFilter) {
		pps.unregisterFilter(proxyFilter);
	}
}

function addProxy(host, port) {
	var lastProxyInfo = null;
	
	if (proxyList.length > 0) {
		lastProxyInfo = proxyList[proxyList.length-1].proxyInfo;
	}
	
	var proxyInfo = pps.newProxyInfo("http", host, parseInt(port), 0, 1, lastProxyInfo);
	
	proxyList.push({proxyInfo: proxyInfo});
	
	youtubeButton.disabled = false;
}

function doUnblock(active) {
	if (active) {
		unblock = true;
		youtubeButton.icon = {"32": "./youtube-unblock-active.png"};
		tabs.activeTab.reload();
	} else {
		unblock = false;
		youtubeButton.icon = {"32": "./youtube-unblock-inactive.png"};
		tabs.activeTab.reload();
	}
}

var youtubeButton = ToggleButton({
	id: "youtube-unblock-button",
	label: "Unblock YouTube",
	icon: {
		"32": "./youtube-unblock-inactive.png",
	},
	disabled: true,
	onChange: youtubeHandleChange
});

function youtubeHandleChange(state) {
	if (state.checked) {
		if (Date.now()-configTimestamp > 30*60*1000) {
			loadAddonConfig(true);
		} else {
			doUnblock(true);
		}
	} else {
		doUnblock(false);
	}
}

pageMod.PageMod({
	include: "*",
	attachTo: ["top"],
	contentScriptFile: [data.url("jquery-1.12.1.min.js"), data.url("notification-script.js")],
	contentScriptWhen: 'end',
	onAttach: function(worker) {
		worker.port.on('getNotifications', function() {
			if (!preferences.disableNotifications) {
				for (var i = 0; i < notifications.length; i++) {
					worker.port.emit("applyNotifications", notifications[i].type, notifications[i].selector, notifications[i].content);
				}
			}
		});
	}
});

exports.main = function (options, callbacks) {
	loadAddonConfig();
	
	if (options.loadReason == 'install') {
		tabs.open('http://www.ytunblocker.com/api/pages/install.html');
	} else if (options.loadReason == 'upgrade') {
		tabs.open('http://www.ytunblocker.com/api/pages/upgrade.html');
	}
};

exports.onUnload = function(reason) {
	unregisterProxyFilter();
	
	if (reason == 'uninstall' || reason == 'disable') {
		tabs.open('http://www.ytunblocker.com/api/pages/uninstall.html');
	}
}
